package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.service.SearchIndexService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 更新搜索索引（阿里云搜索）
 *
 * @author : Charles
 * @date : 2021/4/6
 */
@Api(tags="搜索索引更新API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/search/index")
public class SearchIndexApi {

    private final SearchIndexService searchIndexService;

    /**
     * 初始化（添加）所有索引
     *
     * spi旧接口：/search/manage/init/all
     * @return totalRecord
     */
    @ApiOperation("初始化所有索引")
    @PostMapping("/init")
    public ResponseJson<Integer> initAllIndexes(){
        return searchIndexService.initAllIndexes();
    }

    /**
     * 更新所有商品
     *
     * spi旧接口：/search/manage/update/product/all
     * @return productRecord
     */
    @ApiOperation("更新所有商品")
    @PostMapping("/product/all")
    public ResponseJson<Integer> updateAllProductIndexes(){
        return searchIndexService.updateAllProductIndexes();
    }


    /**
     * 更新所有供应商
     *
     * spi旧接口：/search/manage/update/supplier/all
     * @return shopRecord
     */
    @ApiOperation("更新所有供应商")
    @PostMapping("/shop/all")
    public ResponseJson<Integer> updateAllSupplierIndexes(){
        return searchIndexService.updateAllSupplierIndexes();
    }

    /**
     * 更新所有项目仪器
     *
     * spi旧接口：/search/manage/update/equipment/all
     * @return equipmentRecord
     */
    @ApiOperation("更新所有项目仪器")
    @PostMapping("/equipment/all")
    public ResponseJson<Integer> updateAllEquipmentIndexes(){
        return searchIndexService.updateAllEquipmentIndexes();
    }

    /**
     * 更新所有文章
     *
     * spi旧接口：/search/manage/update/article/all
     * @return articleRecord
     */
    @ApiOperation("更新所有文章")
    @PostMapping("/article/all")
    public ResponseJson<Integer> updateAllArticleIndexes(){
        return searchIndexService.updateAllArticleIndexes();
    }

    /**
     * 根据商品Id更新
     *
     * spi旧接口：/search/manage/update/product
     * @param productId 商品Id
     * @return int
     */
    @ApiOperation("根据商品Id更新")
    @ApiImplicitParam(required = true, name = "productId", value = "商品Id(旧：pid)")
    @PostMapping("/update/product")
    public ResponseJson<Integer> updateProductIndex(Integer productId){
        return searchIndexService.updateProductIndexById(productId);
    }

    /**
     * 根据供应商Id更新
     *
     * spi旧接口：/search/manage/update/supplier
     * @param shopId 供应商Id
     * @return int
     */
    @ApiOperation("根据供应商Id更新")
    @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id(旧：sid)")
    @PostMapping("/update/shop")
    public ResponseJson<Integer> updateSupplierIndex(Integer shopId){
        return searchIndexService.updateSupplierIndexById(shopId);
    }

    /**
     * 根据项目仪器Id更新
     *
     * spi旧接口：/search/manage/update/equipment
     * @param equipmentId 仪器Id
     * @return int
     */
    @ApiOperation("根据仪器Id更新")
    @ApiImplicitParam(required = true, name = "equipmentId", value = "仪器Id(旧：eid)")
    @PostMapping("/update/equipment")
    public ResponseJson<Integer> updateEquipmentIndex(Integer equipmentId){
        return searchIndexService.updateEquipmentIndexById(equipmentId);
    }

    /**
     * 根据文章Id更新
     *
     * spi旧接口：/search/manage/update/article
     * @param articleId 文章Id
     * @return int
     */
    @ApiOperation("根据文章Id更新")
    @ApiImplicitParam(required = true, name = "articleId", value = "文章Id(旧：aid)")
    @PostMapping("/update/article")
    public ResponseJson<Integer> updateArticleIndex(Integer articleId){
        return searchIndexService.updateArticleIndexById(articleId);
    }

    /**
     * 根据商品Id删除
     *
     * spi旧接口：/search/manage/delete/product
     * @param productId 商品Id
     */
    @ApiOperation("根据商品Id删除")
    @ApiImplicitParam(required = true, name = "productId", value = "商品Id(旧：pid)")
    @PostMapping("/delete/product")
    public ResponseJson<Integer> deleteProductIndex(Integer productId){
        return searchIndexService.deleteProductIndexById(productId);
    }

    /**
     * 根据供应商Id删除
     *
     * spi旧接口：/search/manage/delete/supplier
     * @param shopId 供应商Id
     */
    @ApiOperation("根据供应商Id删除")
    @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id(旧：sid)")
    @PostMapping("/delete/shop")
    public ResponseJson<Integer> deleteSupplierIndex(Integer shopId){
        return searchIndexService.deleteSupplierIndexById(shopId);
    }

    /**
     * 根据项目仪器Id删除
     *
     * spi旧接口：/search/manage/delete/equipment
     * @param equipmentId 仪器Id
     */
    @ApiOperation("根据项目仪器Id删除")
    @ApiImplicitParam(required = true, name = "equipmentId", value = "仪器Id(旧：eid)")
    @PostMapping("/delete/equipment")
    public ResponseJson<Integer> deleteEquipmentIndex(Integer equipmentId){
        return searchIndexService.deleteEquipmentIndexById(equipmentId);
    }

    /**
     * 根据文章Id删除
     *
     * spi旧接口：/search/manage/delete/article
     * @param articleId 文章Id
     */
    @ApiOperation("根据文章Id删除")
    @ApiImplicitParam(required = true, name = "articleId", value = "文章Id(旧：aid)")
    @PostMapping("/delete/article")
    public ResponseJson<Integer> deleteArticleIndex(Integer articleId){
        return searchIndexService.deleteArticleIndexById(articleId);
    }

    /**
     * 根据主文档Id删除
     *
     * spi旧接口：/search/manage/delete/main
     * @param id 文档Id
     */
    @ApiOperation("根据主文档Id删除")
    @ApiImplicitParam(required = true, name = "id", value = "文档Id")
    @PostMapping("/delete/main")
    public ResponseJson<Integer> deleteMainIndex(Integer id){
        return searchIndexService.deleteMainIndexById(id);
    }
}
